@@ -26,7 +26,7 @@ def overdamped_langevin(logprob_grad_fn):
     def one_step(rng_key, state: DiffusionState, step_size: float, batch: tuple = ()):
         position, logprob_grad = state
         noise = generate_gaussian_noise(rng_key, position)
-        position = jax.tree_util.tree_multimap(
+        position = jax.tree_util.tree_map(
             lambda p, g, n: p + step_size * g + jnp.sqrt(2 * step_size) * n,
             position,
             logprob_grad,
