@@ -40,15 +40,15 @@ regresion_test_cases = [
         "algorithm": blackjax.hmc,
         "initial_position": {"scale": 1.0, "coefs": 2.0},
         "parameters": {"num_integration_steps": 90},
-        "num_warmup_steps": 3_000,
-        "num_sampling_steps": 2_000,
+        "num_warmup_steps": 1_000,
+        "num_sampling_steps": 3_000,
     },
     {
         "algorithm": blackjax.nuts,
         "initial_position": {"scale": 1.0, "coefs": 2.0},
         "parameters": {},
         "num_warmup_steps": 1_000,
-        "num_sampling_steps": 500,
+        "num_sampling_steps": 1_000,
     },
 ]
 
@@ -70,7 +70,7 @@ class LinearRegressionTest(chex.TestCase):
         return jnp.sum(logpdf)
 
     @parameterized.parameters(itertools.product(regresion_test_cases, [True, False]))
-    def test_linear_regression(self, case, is_mass_matrix_diagonal):
+    def test_window_adaptation(self, case, is_mass_matrix_diagonal):
         """Test the HMC kernel and the Stan warmup."""
         rng_key, init_key0, init_key1 = jax.random.split(self.key, 3)
         x_data = jax.random.normal(init_key0, shape=(1000, 1))
@@ -106,6 +106,69 @@ class LinearRegressionTest(chex.TestCase):
         np.testing.assert_allclose(np.mean(scale_samples), 1.0, atol=1e-1)
         np.testing.assert_allclose(np.mean(coefs_samples), 3.0, atol=1e-1)
 
+    def test_mala(self):
+        """Test the MALA kernel."""
+        rng_key, init_key0, init_key1 = jax.random.split(self.key, 3)
+        x_data = jax.random.normal(init_key0, shape=(1000, 1))
+        y_data = 3 * x_data + jax.random.normal(init_key1, shape=x_data.shape)
+
+        logposterior_fn_ = functools.partial(
+            self.regression_logprob, x=x_data, preds=y_data
+        )
+        logposterior_fn = lambda x: logposterior_fn_(**x)
+
+        warmup_key, inference_key = jax.random.split(rng_key, 2)
+
+        mala = blackjax.mala(logposterior_fn, 1e-5)
+        state = mala.init({"coefs": 1.0, "scale": 2.0})
+        states = inference_loop(mala.step, 10_000, inference_key, state)
+
+        coefs_samples = states.position["coefs"][3000:]
+        scale_samples = states.position["scale"][3000:]
+
+        np.testing.assert_allclose(np.mean(scale_samples), 1.0, atol=1e-1)
+        np.testing.assert_allclose(np.mean(coefs_samples), 3.0, atol=1e-1)
+
+    @parameterized.parameters(regresion_test_cases)
+    def test_pathfinder_adaptation(
+        self,
+        algorithm,
+        num_warmup_steps,
+        initial_position,
+        num_sampling_steps,
+        parameters,
+    ):
+        """Test the HMC kernel and the Stan warmup."""
+        rng_key, init_key0, init_key1 = jax.random.split(self.key, 3)
+        x_data = jax.random.normal(init_key0, shape=(1000, 1))
+        y_data = 3 * x_data + jax.random.normal(init_key1, shape=x_data.shape)
+
+        logposterior_fn_ = functools.partial(
+            self.regression_logprob, x=x_data, preds=y_data
+        )
+        logposterior_fn = lambda x: logposterior_fn_(**x)
+
+        warmup_key, inference_key = jax.random.split(rng_key, 2)
+
+        warmup = blackjax.pathfinder_adaptation(
+            algorithm,
+            logposterior_fn,
+            num_warmup_steps,
+            **parameters,
+        )
+        state, kernel, _ = warmup.run(
+            warmup_key,
+            initial_position,
+        )
+
+        states = inference_loop(kernel, num_sampling_steps, inference_key, state)
+
+        coefs_samples = states.position["coefs"]
+        scale_samples = states.position["scale"]
+
+        np.testing.assert_allclose(np.mean(scale_samples), 1.0, atol=1e-1)
+        np.testing.assert_allclose(np.mean(coefs_samples), 3.0, atol=1e-1)
+
 
 class SGMCMCTest(chex.TestCase):
     """Test sampling of a linear regression model."""
